/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
* @brief Analog Devices SPI ADXL375 Driver
* @defgroup 
*
*/

#include <stdbool.h>
#include "ADXL375_SPI.h"
#include "SEGGER_RTT.h"
#include "nrf_drv_spi.h"
#include "boards.h"
#include "app_error.h"
#include "app_util_platform.h"
#include "nrf_gpio.h"
#include <string.h>

static const nrf_drv_spi_t * p_spi_master_0;

/**@brief Function for initializing the ADXL375 SPI driver.
 *
 * @param[in] nrf_drv_spi_t * m_spi_master    Pointer to spi instance
 *
 */

void ADXL375_SPI_init(const nrf_drv_spi_t * m_spi_master)
{
    p_spi_master_0 = m_spi_master;
    
    // Set the SPI Chip Select pin as output and initial state to high
    nrf_gpio_cfg_output(SPIM0_CS_PIN);
    nrf_gpio_pin_set(SPIM0_CS_PIN);
    
    // Set Data Output Rate to 100Hz (0x0A), see Table 6, p.10 in Datasheet
    ADXL375_SPI_setDataRate(0x0A);
    // Set the ADXL375 in measurement mode by setting the Power Control Register to 0x08 
    ADXL375_SPI_setPowerReg(0x08);
    // Set offset values 
    ADXL375_SPI_setOffsets(0x00,0x00,0x00);
}

/**@brief Function for reading to the ADXL375 registers over SPI.
 *
 * @param[in] address   ADXL375 Register adress    
 * @param[in] p_data    Pointer to rx buffer 
 * @param[in] bytes     Number of bytes to be read 
 *
 */

void ADXL375_SPI_readRegister(uint8_t address, uint8_t * p_data, uint8_t bytes)
{
    uint32_t err_code;
        
    uint8_t m_tx_data_spi;
        
    if(bytes > 1)
    {
        m_tx_data_spi = (0x80|address)|0x40;
    }
    else
    {
        m_tx_data_spi = 0x80|address;
    }
    
    // Pull Chip Select line low
    nrf_gpio_pin_clear(SPIM0_CS_PIN);
    
    err_code = nrf_drv_spi_transfer(p_spi_master_0, &m_tx_data_spi, 
                        bytes, p_data, bytes);
    
    APP_ERROR_CHECK(err_code);
    
    // Set Chipselect line high 
    nrf_gpio_pin_set(SPIM0_CS_PIN);
}

/**@brief Function for writing to the ADXL375 registers over SPI.
 *
 * @param[in] address   ADXL375 Register adress    
 * @param[in] p_data    Pointer to tx buffer 
 * @param[in] bytes     Number of bytes to be written
 *
 */

void ADXL375_SPI_writeRegister(uint8_t reg_addr, uint8_t * p_data, uint8_t bytes)
{
         
    uint32_t err_code;
    
    uint8_t m_tx_data_spi[bytes+1];
      
    memcpy(m_tx_data_spi+1,p_data,bytes);
    
    if(bytes > 1)
    {
        m_tx_data_spi[0] = (reg_addr|0x40);
    }
    else
    {
        m_tx_data_spi[0] = reg_addr;
    }
    
    // Pull Chip Select line low
    nrf_gpio_pin_clear(SPIM0_CS_PIN);
     
    err_code = nrf_drv_spi_transfer(p_spi_master_0, m_tx_data_spi, 
                        sizeof(m_tx_data_spi), NULL, 0);
    
    APP_ERROR_CHECK(err_code);
    
    // Set Chip Select line high 
    nrf_gpio_pin_set(SPIM0_CS_PIN);
}

/**@brief Function for setting the data rate of the ADXL375 .
 *
 * @param[in] rate      Data rate in hex (refer the ADXL375 datasheet for available rates)   
 *
 */

void ADXL375_SPI_setDataRate(uint8_t rate)
{
   // SEGGER_RTT_printf(0,"Setting Data Rate Register: %x \n", rate);
    
    uint8_t m_rx_data_spi[2];
    
    ADXL375_SPI_writeRegister(0x2C,&rate,1);
    
    // Read the register and check if it was set correctly
    ADXL375_SPI_readRegister(0x2C,m_rx_data_spi, sizeof(m_rx_data_spi));

   
     if(m_rx_data_spi[1] != rate)
    { 
        SEGGER_RTT_WriteString(0,"Failed to set data rate!\n");
    }
    else
    {
        SEGGER_RTT_WriteString(0,"Data rate set sucessful!\n");
    }
}

/**@brief Function for setting the POWER_CTL register of the ADXL375 .
 *
 * @param[in] reg_val       Value to be written the register   
 *
 */

void ADXL375_SPI_setPowerReg(uint8_t reg_val)
{    
    SEGGER_RTT_printf(0,"Setting Power Control Register: %x \n", 0x8);
    
    uint8_t m_tx_data_spi =  reg_val;
    uint8_t m_rx_data_spi[2];
    
    ADXL375_SPI_writeRegister(0x2D, &m_tx_data_spi,1);
    
    ADXL375_SPI_readRegister(0x2D, m_rx_data_spi, sizeof(m_rx_data_spi));
    
     if(m_rx_data_spi[1] != reg_val)
    { 
        SEGGER_RTT_WriteString(0,"Failed to set Power Control Register !\n");
    }
    else
    {
        SEGGER_RTT_WriteString(0,"Power Control Register set sucessful!\n");
    }
}

/**@brief Function for reading the data registers of the ADXL375 .
 *
 * @param[in] rx_buffer     Pointer to rx buffer 
 * @param[in] rx_buf_size   Size of rx buffer  
 * @param[in] x_val         Pointer to x value  
 * @param[in] y_val         Pointer to y value    
 * @param[in] z_val         Pointer to z value    
 *
 */

void ADXL375_SPI_readAccelerometerData( uint8_t * rx_buffer, uint8_t rx_buf_size, int16_t *x_val, int16_t *y_val, int16_t *z_val)
{  
    
     ADXL375_SPI_readRegister(0x32,rx_buffer,rx_buf_size);
    
    for(int i = 0; i<7; i++)
    {
        SEGGER_RTT_printf(0,"Data[%d]: %x \n",i, rx_buffer[i]);
    }    
    
    *x_val = (rx_buffer[2]<<8) | rx_buffer[1];
    *y_val = (rx_buffer[4]<<8) | rx_buffer[3];
    *z_val = (rx_buffer[6]<<8) | rx_buffer[5];
    
    SEGGER_RTT_printf(0,"X:  %d  Y:  %d  Z:  %d  \n",*x_val,*y_val,*z_val );        
}


/**@brief Function for setting the OFSn register of the ADXL375, n = [X,Y,Z] .
 *
 * @param[in] x_offset       X-axis offset
 * @param[in] y_offset       Y-axis offset
 * @param[in] z_offset       Z-axis offset
 *
 */

void ADXL375_SPI_setOffsets(uint8_t x_offset, uint8_t y_offset, uint8_t z_offset )
{ 
    uint8_t m_tx_data_spi[3] = {x_offset, y_offset, z_offset};
    uint8_t m_rx_data_spi[4];
    
    ADXL375_SPI_writeRegister(0x1E,m_tx_data_spi,sizeof(m_tx_data_spi));
    
    ADXL375_SPI_readRegister(0x1E,m_rx_data_spi,sizeof(m_rx_data_spi));
    
 
    if(m_rx_data_spi[1] == x_offset &&
        m_rx_data_spi[2] == y_offset &&
        m_rx_data_spi[3] == z_offset)
    {
        SEGGER_RTT_printf(0,"Offsets set sucessfully!  \n");
    }
    else
    {
        SEGGER_RTT_printf(0,"Setting offsets failed! X: %u Y: %u Z: %u \n", m_rx_data_spi[1], m_rx_data_spi[2], m_rx_data_spi[3]);
    }
}

